#ifndef TIDCLASSH
#define TIDCLASSH

#include <vector>
#include <list>
#include <map>
#include <Rectangle.h>
#include <string>
#include <TidIO.h>
#include <fstream>
#include <pthread.h>

#include <sys/types.h>
#include <sys/stat.h>


#ifdef WithDMT
#include <html/writer.hh>
#include <html/table.hh>
#include <html/document.hh>
#include <html/color.hh>
#include <html/text.hh>
#endif

class TidEventsDef {

 public:

  bool NormalizeCost;

  double BPP;

  string desc; // description

  unsigned int NTrigger; // number of detections

  double maxCost; // maximum cost for detection

  TidCompositeRectangle *rect;
  // TidAirplane *air;

  TidEventsDef (string d) {NTrigger=0; desc=d; rect=NULL; NormalizeCost = 0; BPP = 0.5;}
  ~TidEventsDef () {delete rect;}

  void SetmaxAboveThrStop (unsigned int);

  void GotTrigger () {NTrigger++;}
};

typedef list<TidEventsDef *> ListTidEventsDef;
typedef list<string> ListString;
typedef vector<bool> VectorBool;
typedef list<VectorBool> ArrayBool;
typedef vector<double> VectorDouble;

class TidClass {

 protected:
  bool DoTrigger; // set to 1 to send stuff to trigger manager

 private:

  bool DoDumpCat;
  string DumpCatFile;

  pthread_t RLooper;

  static const unsigned SleepTime = 1;
  static const unsigned CatRefreshTime = 60; // in units of SleepTime

  string htmlFile; // html file output
  bool HtmlDef;

  string ReportFile; // html file for status report
  bool DoReport;

#ifdef WithDMT
  std::ofstream* htmlStream;
  html::writer* htmlWriter;
  html::table* htmlTable;
  html::document* htmlDocu;
#endif

  bool ASCIIDef;
  string ASCIIFile;
  std::ofstream* ASCIIStream;

  string catPath; // catalog path

  list<string> catFiles; // list of files in catalog
  list<time_t> catTimes; // last data modification

  double TimeResolution; // time resolution of the server, inverse bandwidth
  double T0; // GPS time of time index 0 (from data server)

  double GraceTime;

  unsigned int UpTime, UpTimeRef, UpTimeMax; // uptime in seconds

  list<string> source_;

  list<string> chnames_;
  list<unsigned int> cid_, fs_;

  list<TidIO*> tio;  // communication server

  map<unsigned int, unsigned int> cidid;
  map<string, unsigned int> namecid;

  // the following are organized according to cidid
  vector<ListString> eventFiles;
  vector<ListTidEventsDef> events;
  vector<VectorDouble> eventFirstTime;
  vector<ArrayBool> data;
  vector<double> FirstTime; 
  vector<double> LastTime, TimeWidth;
  vector<bool> FirstTimeDefined;
  vector<unsigned int> NFreq;

 public:
  TidClass(int argc, const char *argv[]);
  ~TidClass();

  void InitIO(string);
  void InitCatalog();

  void ReadCatFile(string);

  bool Run(); // returns 0 when done, 1 otherwise

  void GetData();

  void Classify();

  void RereadCatalog ();

  void Report ();

  void DumpCat(const char *);
};

#endif
