#ifndef TIDEVENTH
#define TIDEVENTH

#include <string>
#include <vector>
#include <list>

typedef std::vector<bool> VectorBool;
typedef std::list<VectorBool> ArrayBool;

class TidEvent {

 protected:

  double alpha; // cost factor

  //  double maxT; // max duration (seconds) of an event

  std::vector<double> constants;

  std::vector<char> varname;
  std::vector<double> varlb, varub; // lower & upper bounds on variable
  std::vector<double> dvar; // step size

  std::vector<double> addFactor, mulFactor; // additive and multiplicative factors
  std::vector<unsigned int> params; // map parameters to constants or variables+1000

 public:

  TidEvent(double alpha, unsigned int argc, std::vector<std::string>& argv);

  virtual ~TidEvent() {};

  //  double Cost(ArrayBool *); // returns minimum cost from data

  virtual double CostFunction(ArrayBool *, std::vector<double>*) = 0;

  virtual void Delete(ArrayBool *, std::vector<double>*) = 0;

  virtual double Area(std::vector<double>*, double) = 0;

  //  double GetMaxT () {return maxT;}

  double GetParamMin (unsigned int i);

  double GetParamMax (unsigned int i);

  double GetParam(unsigned int i) {return params[i];}

  std::vector<unsigned int> GetParams () {return params;}

  double GetConstant(unsigned int i) {return constants[i];}

  std::vector<char>* GetVarName () {return &varname;}

  std::vector<double>* GetVarLB() {return &varlb;}
  std::vector<double>* GetVarUB() {return &varub;}
  std::vector<double>* GetDVar() {return &dvar;}

  double GetAdd(unsigned int i) {return addFactor[i];}
  double GetMul(unsigned int i) {return mulFactor[i];}
};

#endif

